#!/bin/bash	


MISSING_CONF=1
MD5_MISSMATCH=2
COULD_NOT_FETCH_FILE=3
COULD_NOT_UNPACK=4
MISSING_FUNCTIONS=5
COULD_NOT_TAR=6
FIX_CONF=7
NEED_SEC=8
BROKEN=9

function parse_arguments {
	needhelp=yes
    while test "$#" -gt "0" ;
    do
	    needhelp=no
	    case "$1" in 
		    
		install|-i|--install) 
			needinstall=yes
			
			if test -f "$2"; then  
				distfile=$2
				shift
			elif test "$2" = "f" -o "$2" = "force";then
				force_install=yes
				shift
			elif test "$2" != "" ; then
				test "$3" = "" && error $NEED_SEC
				cd $ports_dir/$2/$3	
			fi
		;;
		
		install-list|-l|--install-list) 
			needinstalllist=yes
			installlist_filename="$2"
			shift
		;;
		
		updated|-u|--show-updated) 
			needupdated=yes
		
			if test "$2" = "r" -o "$2" = "recursive"; then
				rec_updated=yes
				shift
			fi

		;;


		only-package|-p|--only-package) 
			needinstall=yes
			onlyneedpackage=yes
			force_install=yes
			
			if test "$2" = "f" -o "$2" = "force"; then
				force_install=yes
				shift
			fi

		;;

		remove|-r|--remove) 
			needremove=yes
		;;
		search|-s|--search) 
			needsearch=yes
			topic=$2
			shift
			
		;;
		
		cleanup|-c|--clean-up) 
			
			if test "$2" = "r" -o "$2" = "recursive"; then
				rec_cleanup=yes
				shift
			fi

			if test "$2" = "rc";  then
				cd `dirname $3`
				echo cleaning up `pwd`
				shift
				shift
			fi
			needcleanup=yes
		;;
		homepage|-h|--homepage)
			needhome=yes
		;;
		
		show-backends|-s|--show-backends)
			needsb=yes
		;;
		
		*)
			needhelp=yes
		;;
		
	    esac
	    shift
    done
}

function clean_up {
pkg_filename=`test -e ./conf && get_xml ./conf port download filename` || error -1 
		rm -rf build/ patches/ heap/ 2>/dev/null
		rm -f $distfiles/$pkg_filename 2>/dev/null
}

function make_skel {

	olddir=$PWD
	 
	cd $portdir
	test -d build || mkdir build
	test -d patches || mkdir patches
	test -d heap || mkdir heap
	
	rm -rf heap/* 
	cd heap/
	mkdir bin boot dev etc lib opt sbin usr var 
	cd usr
	mkdir X11R6 info libexec sbin bin doc etc include lib local man share 
	cd local 
		mkdir bin sbin lib include
		mkdir man 
			cd man 
			mkdir man1 man2 man3 man4 man5 man6 man7 man8
		cd ..
	cd ..
	cd share 
	mkdir man 
	cd man 
	mkdir man1 man2 man3 man4 man5 man6 man7 man8
	cd $olddir
}

function clean_up_heap {
	for a in `find $portdir/heap/ -type d` ; do
		rmdir "$a" > /dev/null 2>&1
	done
	
	for a in `find $portdir/heap/ -type d` ; do
		rmdir "$a" > /dev/null 2>&1
	done

}

function build {
#build the package 
# configure script called here ($pkg_conf)
# and make is called here ($pkg_make)

$pkg_conf

export compile_arch=`arch`

if [ "$pkg_no_march" = "" ] ; then
	if [ "$force_march" = "" ] ; then 
		echo 'march='`arch`
		export compile_arch=`arch`
		export CFLAGS='march='`arch`
	else	
		echo 'march='$force_march
		export compile_arch=$force_march
		export CFLAGS=$CFLAGS' march='$force_march
	fi
fi

$pkg_make

test "$?" = 0 || error -1
export CFLAGS=''
}

function source_conf {
	test -e "./conf" || error $MISSING_CONF
	# source ./conf | old style -> ugly :) 
	# new xml stuff
	test "`get_xml ./conf port information broken`" = "true" && error $BROKEN
	
	test "$exec_prefix" = "" && export exec_prefix=$install_prefix
	test "$sysconfdir" = "" && export sysconfdir=$install_prefix/etc
	test "$localstatedir" = "" && export localstatedir=$install_prefix/var
	
	export pkg_url=`get_xml ./conf port download mirror`
	export pkg_homepage=`get_xml ./conf port information homepage`
	export pkg_mirror_subdir=`get_xml ./conf port download subdir`
	export pkg_mirrors=`get_xml ./conf port download mirror`
	
	
		#export pkg_category=`get_xml ./conf port category`
		#export pkg_maintainer_name=`get_xml ./conf port maintainer realname`
		#export pkg_maintainer_email=`get_xml ./conf port maintainer email`
		#export pkg_description=`get_xml ./conf -br port description`
		
		export pkg_dir_intar=`get_xml ./conf port build decompressed-directory`
		export pkg_unzip=`get_xml ./conf port build decompress`
		export pkg_filename=`get_xml ./conf port download filename `
		export pkg_name=`get_xml ./conf port information name`
		
		export pkg_minor=`get_xml ./conf port information minor`
		export pkg_major=`get_xml ./conf port information major`
		export pkg_version="$pkg_major.$pkg_minor"
		
		export pkg_depends=`get_xml ./conf port build depends`
		
		export pkg_conf_args=`get_xml ./conf -br port build configure arguments`
		
		case "`get_xml ./conf -br port build style`" in 
		
		"gnu-destdir")
			export pkg_overwrite_make=`get_xml ./conf -br port build make`
			export pkg_make_install="make install DESTDIR=$portdir/heap"
			export pkg_conf="./configure --localstatedir=$localstatedir --prefix=$install_prefix --exec-prefix=$exec_prefix --sysconfdir=$sysconfdir"
		;;
		"gnu-prefix")
			export pkg_make="make"
			export pkg_make_install="make install prefix=$portdir/heap/$install_prefix exec_prefix=$portdir/heap/$exec_prefix sysconfdir=$protdir/heap/$sysconfdir localstatedir=$portdir/heap/$localstatedir"
			export pkg_conf="./configure --localstatedir=$localstatedir --prefix=$install_prefix --exec-prefix=$exec_prefix --sysconfdir=$sysconfdir"
			;;
		*) 
			export pkg_make=`get_xml ./conf -br port build make`
			export pkg_make_install=`get_xml ./conf -br port build install`
			export pkg_conf=`get_xml ./conf -br port build configure script`
			;;
		esac
		
		
		export pkg_conf="$pkg_conf $pkg_conf_args"
		
		export pkg_no_march=`get_xml ./conf port no_march`
		export pkg_build_option=`get_xml ./conf -wt port build_option`
      		export pkg_name_version=$pkg_name-$pkg_version
		
		if test "$pkg_mirrors" = " "; then
			export pkg_mirrors="$portdir/mirrors"
		fi
		
		test "$pkg_depends" = " " && pkg_depends=""
	
	
		for a in `echo $pkg_build_option | tr ':' '\n'` ; do
			echo not workin anymore
			exit
			for opt in `get_xml ./conf -getbyc port build_options "$a" configure | tr ':' '\n' ` ; do
				inst_it=`get_xml ./conf port build_options "$a" default`
				if test "$inst_it" != "true" ; then 
					continue
				fi

				if test "$run" = "true" ; then
					run="false";
					opt_value=$opt;
					pkg_conf=`echo -e $pkg_conf | sed -e s/$opt_name=/$opt_name=$opt_value/g`
				else
					run="true";
					opt_name=$opt;
				fi
			done
		done
}			

function get_file {
	origdir=`pwd`
	if test -f $pkg_mirrors ; then 
		cd "$distfiles"
		
		for url in `cat $pkg_mirrors`; do 
						
			#if test -e "$portdir/files/$pkg_filename"; then 
			#	cd ..
			#	return 0
			#fi
			
			if test "$pkg_mirror_subdir" = " " ; then
				$get_command "$url/$pkg_filename"
			else
				$get_command "$url$pkg_mirror_subdir/$pkg_filename"
			fi
			test "$?" = "0" && break
		done
	else
		cd "$distfiles"
		$get_command "$pkg_url/$pkg_filename"
		
		return 0
	fi
	cd $origdir
}

function checksum {
	if test -e "./md5"  ; then 
		origdir=`pwd`
		cd $distfiles
		md5sum -c $portdir/md5
		if test "$?" != 0 ; then 
			error $MD5_MISSMATCH
		fi
		cd $origdir
	fi

}

function solve_deps {
	stillrunme=1
	if test "$pkg_depends" != ""  
	then
		pkg_depends="$pkg_depends:end"
		origdir=$PWD
		while test "$stillrunme" -ne 0 
		do
			dep=`echo $pkg_depends | cut -d: -f $stillrunme`
			let stillrunme=$stillrunme+1	
			if test "$dep" = "end" 
			then 
				stillrunme=0
			else 
				echo "$name installing $dep as dep from $pkg_name"
				cd $ports_dir/$dep
				/bin/sh --login $0 --install
				cd $origdir
			fi
		done
	else
		echo "$name No package dep's"
	fi
}

function get_patches {
	sec=`pwd | sed -e s\|$ports_dir\|\|g`
	cd $portdir/patches
	$get_command $patch_host/$patch_dir/$sec/*
	cd ..
}

function apply_patches {
	for patch in `ls $portdir/patches/`; do
		patch < $portdir/patches/$patch 
	done
}

function show_backends {
	echo $name using $ports_backend as backend
	printf "Backends:\n"
	printf "\tinstmon: a module that works with instmon, bit buggy but lfs :)\n"
	printf "\tslackware: a backend for install_pkg,remove_pkg and make_pkg\n"
	printf "\trpm: not working yet\n"
	printf "\tdpkg: not working yet\n"
}

function get_xml {
	#we have to optimize this and move it to a single script!
	name=$1
	shift
	
	test "$install_prefix" = "" && install_prefix="/usr/local/"
	
	const="s:#PORTDIR#:$portdir:g;\
	s:#GNU_MIRRORS#:$ports_dir/internals/mirrors/gnu:g;\
	s:#SF_MIRRORS#:$ports_dir/internals/mirrors/sf:g;\
	s:#PREFIX#:$install_prefix:g;\
	s:#EXEC_PREFIX#:$exec_prefix:g;\
	s:#SYSCONFDIR#:$sysconfdir:g;\
	s:#TO#:\>:g;\
	s:#HEAP#:$portdir/heap/:g;"
	
	if test "$1" = "-br" ; then 
		
		shift
		cat $name | sed -e 's/^[[:space:]]*//' |tr '\n' ':' |\
		sed  -e "s:`xml_build_regex $@`:\1:g;s:<.*>::g" -e "$const"| tr ':' '\n' 
		
	elif test "$1" = "-getargs" ; then
		
		shift
		cat $name | sed -e 's/^[[:space:]]*//' |tr '\n' ':' |\
		sed  -e "s:`xml_build_arg_regex $@`:\1:g;s:<.*>::g" -e "$const"| tr ':' '\n' 
	
	elif test "$1" = "-getbyc" ; then
		
		shift
		cat $name | sed -e 's/^[[:space:]]*//' |tr '\n' ':' |\
		sed -e "s:`xml_build_argc_regex $@`:\1\:\2:g" -e "$const"| tr ':' '\n' 
	
	elif test "$1" = "-wt" ; then
		shift
		cat $name | sed -e 's/^[[:space:]]*//' |tr '\n' ':' |\
		sed -e "s:`xml_build_regex $@`:\1:g;s:<.*>::g" -e "$const"| tr ':' '\n' 

	else
		cat $name | sed -e 's/^[[:space:]]*//' |tr '\n' ' ' |\
		sed  -e "s:`xml_build_regex $@`:\1:g;s:<.*>::g" -e "$const"
	fi
}

function xml_build_arg_regex {
	while test "$#" -gt "1" ; do
		pre_regex="$pre_regex.*<$1[^>]*>"
		post_regex="</$1>.*$post_regex"
		shift
	done
	
	regex=".*<$1 \(.[^>]*\)>.*</$1>.*"

	regex=$pre_regex$regex$post_regex
	echo $regex

}

function xml_build_argc_regex {

	while test "$#" -gt "1" ; do
		pre_regex="$pre_regex.*<$1[^>]*>"
		post_regex="</$1>.*$post_regex"
		shift
	done
	
	regex=".*<$1 \(.[^>]*\)>\(.*\)</$1>.*"

	regex=$pre_regex$regex$post_regex
	echo $regex
}



function xml_build_regex {
	
	regex="\(.[^>]*\)"

	while test "$#" -gt "0" ; do
		pre_regex="$pre_regex.*<$1*>"
		post_regex="</$1>.*$post_regex"
		shift
	done

	regex=$pre_regex$regex$post_regex
	echo $regex 
}

function install_pkg_list {
	for a in `cat $1`; do
		cd $a 
		pkg_ports -i 
		cd $ports_dir
	done
}


function search {
	if test -e "$1" ; then
		olddir=`pwd`
		
		test -e ./conf || error $MISSING_CONF
		pkg_description=`get_xml $1 port information description`
		pkg_name=`get_xml $1 port infotmation name`
				
		if test "`echo $pkg_name $pkg_description | sed -e "s:.*$2.*::g"`" = "" ; then
			cd `dirname $1`

			pkg_name=`get_xml ./conf port information name`
			pkg_minor=`get_xml ./conf port information minor `
			pkg_major=`get_xml ./conf port information major`
			pkg_version="$pkg_major.$pkg_minor"
			pkg_depends=`get_xml ./conf port build depends`
			pkg_description=`get_xml ./conf -br port information description`
	 		pkg_homepage=`get_xml ./conf port information homepage`
			pkg_category=`get_xml ./conf port information index` 
			pkg_maintainer_name=`get_xml ./conf port maintainer-id`
		
			location=`echo $1 | sed -e 's:$ports_dir::g;s:conf::g;s:\./::g'`

			
			printf "\n"
			printf "Port : %s (%s)\n" "$pkg_name" "$pkg_version"
			printf "Maint: %s\n" "$pkg_maintainer_name" 
			printf "Deps : %s\n" "$pkg_depends"
			printf "Index: %s\n" "$pkg_category"
			printf "Path : %s\n" "$location"
			printf "URL  : %s\n" "$pkg_homepage" 
			printf "Info : %s\n" "$pkg_description" 
			printf "\n"
			cd $olddir
		fi
	fi
}

function is_upto_date {
	#echo processing $1
	
	if test -e "$1"; then
		cd `dirname $1`
	fi
	
	test -e ./conf || error $MISSING_CONF
	pkg_name=`get_xml ./conf port information name`
	pkg_version=`get_xml ./conf port information minor`
	pkg_version=$pkg_version"."`get_xml ./conf port information major`
		
	sh $ports_dir/internals/backends/$ports_backend/updated "$pkg_name" "$pkg_version"
	
	cd $portdir
}

function error { 
	printf "!E!%s error: " $name
	case $1 in 
	
		$MISSING_CONF) 
			echo "missing ./conf file"
		;;

		$MISSING_FUNCTIONS)
			echo "missing functions.sh file"
		;;
		$NEED_SEC)
			echo "need section also"
		;;

		$COULD_NOT_UNPACK)
			echo "could not extract"
		;;
		$COULD_NOT_TAR)	
			echo "could not tar"
		;;
		$FIX_CONF)	
			echo "please customize /etc/ports"
		;;
		$BROKEN)	
			echo "port is broken at the moment"
		;;

		*)
			echo "undefined"
		;;
	esac
	exit $1
}
